Imports System.ComponentModel

<Serializable(), _
 TypeConverter(GetType(AddressCollectionConverter))> _
Public Class AddressCollection
    Inherits CollectionBase

    Default Public Property Item(ByVal index As Integer) As StreetAddress
        Get
            Return CType(List(index), StreetAddress)
        End Get
        Set(ByVal Value As StreetAddress)
            List(index) = Value
        End Set
    End Property

    Public Sub Add(ByVal street_address As StreetAddress)
        List.Add(street_address)
    End Sub

    Public Function IndexOf(ByVal value As StreetAddress) As Integer
        Return List.IndexOf(value)
    End Function

    Public Sub Insert(ByVal index As Integer, ByVal value As StreetAddress)
        List.Insert(index, value)
    End Sub

    Public Sub Remove(ByVal value As StreetAddress)
        List.Remove(value)
    End Sub

    Public Function Contains(ByVal value As StreetAddress) As Boolean
        Return List.Contains(value)
    End Function

    Protected Overrides Sub OnInsert(ByVal index As Integer, ByVal value As Object)
        If Not value.GetType() Is GetType(StreetAddress) Then
            Throw New ArgumentException("Value must be of type StreetAddress.", "value")
        End If
    End Sub

    Protected Overrides Sub OnRemove(ByVal index As Integer, ByVal value As Object)
        If Not value.GetType() Is GetType(StreetAddress) Then
            Throw New ArgumentException("Value must be of type StreetAddress.", "value")
        End If
    End Sub

    Protected Overrides Sub OnSet(ByVal index As Integer, ByVal oldValue As Object, ByVal newValue As Object)
        If Not newValue.GetType() Is GetType(StreetAddress) Then
            Throw New ArgumentException("New value must be of type StreetAddress.", "newValue")
        End If
    End Sub

    Protected Overrides Sub OnValidate(ByVal value As Object)
        If Not value.GetType() Is GetType(StreetAddress) Then
            Throw New ArgumentException("Value must be of type StreetAddress.", "value")
        End If
    End Sub

    Public Sub New()
    End Sub
    Public Sub New(ByVal txt As String)
        Dim addresses() As String = txt.Split(New Char() {";"c})

        For Each address_text As String In addresses
            Try
                Me.List.Add(New StreetAddress(address_text))
            Catch
                Throw New InvalidCastException( _
                    "Invalid StreetAddress serialization '" & _
                    address_text & "'")
            End Try
        Next address_text
    End Sub

    Public Overrides Function ToString() As String
        Dim txt As String
        For Each street_address As StreetAddress In MyBase.List
            txt &= ";" & street_address.ToString()
        Next street_address

        ' Remove the leading ;.
        If txt.Length > 0 Then txt = txt.Substring(1)

        Return txt
    End Function
End Class

Public Class AddressCollectionConverter
    Inherits TypeConverter

    Public Overloads Overrides Function CanConvertFrom(ByVal context As System.ComponentModel.ITypeDescriptorContext, ByVal sourceType As System.Type) As Boolean
        If (sourceType.Equals(GetType(String))) Then Return True
        Return MyBase.CanConvertFrom(context, sourceType)
    End Function

    Public Overloads Overrides Function CanConvertTo(ByVal context As System.ComponentModel.ITypeDescriptorContext, ByVal destinationType As System.Type) As Boolean
        If (destinationType.Equals(GetType(String))) Then Return True
        Return MyBase.CanConvertTo(context, destinationType)
    End Function

    Public Overloads Overrides Function ConvertFrom(ByVal context As System.ComponentModel.ITypeDescriptorContext, ByVal culture As System.Globalization.CultureInfo, ByVal value As Object) As Object
        If (TypeOf value Is String) Then
            Dim txt As String = CType(value, String)
            Return New AddressCollection(txt)
        Else
            Return MyBase.ConvertFrom(context, culture, value)
        End If
    End Function

    Public Overloads Overrides Function ConvertTo(ByVal context As System.ComponentModel.ITypeDescriptorContext, ByVal culture As System.Globalization.CultureInfo, ByVal value As Object, ByVal destinationType As System.Type) As Object
        ' String.
        If (destinationType.Equals(GetType(String))) Then Return value.ToString()

        ' Other.
        Return MyBase.ConvertTo(context, culture, value, destinationType)
    End Function

    Public Overloads Overrides Function GetPropertiesSupported(ByVal context As ITypeDescriptorContext) As Boolean
        Return True
    End Function

    Public Overloads Overrides Function GetProperties(ByVal context As ITypeDescriptorContext, ByVal value As Object, ByVal Attribute() As Attribute) As PropertyDescriptorCollection
        Return TypeDescriptor.GetProperties(value)
    End Function
End Class
